import logging
from typing import List

CRITICAL = 50
FATAL = CRITICAL
ERROR = 40
WARNING = 30
WARN = WARNING
INFO = 20
DEBUG = 10
NOTSET = 0


def get_inner_detail(data_list: List[tuple], sep: str = ":"):
    """
        For remove repeat
    """
    detail = ""
    last_cond = False
    for cond, data in data_list:
        if cond:
            if last_cond:
                detail += sep
            detail += str(data)
        last_cond = cond
    return detail


def get_outer_detail(data_list: [str], sep: str = " - "):
    """
        For remove repeat
    """
    detail = ""
    last_cond = False
    for data in data_list:
        cond = data != ""
        if cond and last_cond:
            detail += sep
        detail += str(data)
        last_cond = cond
    if detail != "":
        return "[ " + detail + " ] "
    else:
        return detail


class BaseLogHandler(logging.Handler):
    """
        A handler class which send data to anywhere
    """
    EXTRA_FIELDS = {"name", "msg", "args", "levelname", "levelno", "pathname", "filename", "module", "exc_info",
                    "exc_text", "stack_info", "lineno", "funcName", "created", "msecs", "relativeCreated", "thread",
                    "threadName", "processName", "process", "message",
                    "logger_data", "extra_data", "short_message", "app_name", "host_name"}
    LOGGER_DATA = {"name", "levelname", "levelno", "pathname", "filename", "module",
                   "lineno", "funcName", "created", "msecs", "relativeCreated", "thread",
                   "threadName", "processName", "process"}

    def __init__(self, app_name: str = None, host_name: str = None, extra_data: dict = None):
        super().__init__()
        self.app_name = app_name
        self.host_name = host_name
        self.extra_data = extra_data

    def __get_logger_data(self, record: "logging.LogRecord"):
        """
            Get some log data
        """
        return {key: getattr(record, key, None) for key in self.LOGGER_DATA}

    def _get_send_data(self, record: logging.LogRecord):
        send_data = {
            "logger_data": self.__get_logger_data(record=record),
            "short_message": record.getMessage(),
        }
        if self.app_name is not None:
            send_data["app_name"] = self.app_name
        if self.host_name is not None:
            send_data["host_name"] = self.host_name
        if self.extra_data is not None:
            send_data["extra_data"] = self.extra_data

        for key, value in record.__dict__.items():
            if key not in self.EXTRA_FIELDS:
                if type(value) not in (bool, int, list, dict, str, float):
                    value = str(value)
                send_data[key] = value
        return send_data



import logging
from typing import Dict, Tuple

import pytz
from termcolor import colored, cprint
# noinspection PyProtectedMember
from termcolor._types import Color

from .base import NOTSET, CRITICAL, ERROR, WARNING, INFO, DEBUG, get_inner_detail, get_outer_detail


class ColorfulStreamHandler(logging.StreamHandler):
    """
        A handler class which write stram handler.
    """
    LEVEL_COLOR_DATA: Dict[int, Tuple[str, Color]] = {
        NOTSET: ("NOTSET   ", "cyan"),
        CRITICAL: ("CRITICAL ", "light_red"),
        ERROR: ("ERROR    ", "red"),
        WARNING: ("WARNING  ", "blue"),
        INFO: ("INFO     ", "light_cyan"),
        DEBUG: ("DEBUG    ", "light_magenta"),
    }

    def __init__(self, colorful: bool = True, show_level: bool = True,
                 show_datetime: bool = True, datetime_color: Color = "green",

                 show_logger_name: bool = True, file_depth: int = 1, show_file: bool = True,
                 show_line: bool = True, show_func: bool = True, file_color: Color = "magenta",

                 show_process_name: bool = False, show_process_id: bool = False,
                 show_thread_name: bool = False, show_thread_id: bool = False,
                 process_thread_color: Color = "blue",

                 is_jalali: bool = True, timezone: str = None, datetime_format: str = "%Y-%m-%d %H:%M:%S"):
        super().__init__()
        self.colorful = colorful
        self.show_level = show_level

        self.show_datetime = show_datetime
        self.datetime_color = datetime_color

        self.show_logger_name = show_logger_name
        self.file_depth = file_depth
        self.show_file = show_file
        self.show_line = show_line
        self.show_func = show_func
        self.file_color = file_color

        self.show_process_name = show_process_name
        self.show_process_id = show_process_id
        self.show_thread_name = show_thread_name
        self.show_thread_id = show_thread_id
        self.process_thread_color = process_thread_color

        self.is_jalali = is_jalali
        self.timezone = timezone
        self.datetime_format = datetime_format

    def get_color_text(self, text, color: Color):
        """
            Get Color Text
        """
        if self.colorful:
            return colored(text, color=color, force_color=True)
        else:
            return str(text)

    def get_datetime(self):
        """
            Get Time
        """
        if self.is_jalali:
            try:
                # noinspection PyPep8Naming
                from persiantools.jdatetime import JalaliDateTime as datetime
            except ImportError:
                cprint("Install persiantools library with pip install", color="red", force_color=True)
                from datetime import datetime
        else:
            from datetime import datetime
        tz = None
        if self.timezone:
            tz = pytz.timezone(self.timezone)
        return get_inner_detail([
            (self.show_datetime, datetime.now(tz=tz).strftime(self.datetime_format))
        ])

    def get_process_thread(self, record):
        """
            Process and thread detail
        """
        process_detail = get_inner_detail([
            (self.show_process_name, record.processName),
            (self.show_process_id, record.process)
        ])
        thread_detail = get_inner_detail([
            (self.show_thread_name, record.threadName),
            (self.show_thread_id, record.thread)
        ])
        return get_outer_detail([process_detail, thread_detail])

    def get_file_detail(self, record: logging.LogRecord):
        """
            Get File and line detail
        """
        logger_name = get_inner_detail([
            (self.show_logger_name, record.name),
        ])
        file_detail = get_inner_detail([
            (self.show_file, "/".join(record.pathname.split("/")[-self.file_depth:])),
            (self.show_line, record.lineno),
        ])
        function_detail = get_inner_detail([
            (self.show_func, record.module + "()"),
        ])

        return get_outer_detail([logger_name, file_detail, function_detail])

    def get_level(self, record):
        """
            Get Level
        """
        return get_inner_detail([
            (self.show_level, self.LEVEL_COLOR_DATA.get(record.levelno, ("",))[0])
        ])

    def get_level_color(self, record) -> Color:
        """
            Get Level Color
        """
        default: Tuple[str, Color] = ("", "yellow")
        color_data: Tuple[str, Color] = self.LEVEL_COLOR_DATA.get(record.levelno, default)
        return color_data[1]

    def format(self, record):
        """
        Format the specified record.

        If a formatter is set, use it. Otherwise, use the default formatter
        for the module.
        """
        # noinspection PyUnresolvedReferences,PyProtectedMember
        formatter = logging._defaultFormatter
        record.message = record.getMessage()

        color = self.get_level_color(record)

        color_datetime = self.get_color_text(self.get_datetime(), color=self.datetime_color)
        color_level = self.get_color_text(self.get_level(record), color=color)
        color_process_thread = self.get_color_text(self.get_process_thread(record),
                                                   color=self.process_thread_color)
        color_file_detail = self.get_color_text(self.get_file_detail(record), color=self.file_color)
        color_text = self.get_color_text(record.getMessage(), color=color)

        message = f"{color_datetime} {color_level}{color_process_thread}{color_file_detail}{color_text}"

        if record.exc_info:
            # Cache the traceback text to avoid converting it multiple times
            # (it's constant anyway)
            if not record.exc_text:
                record.exc_text = formatter.formatException(record.exc_info)
        if record.exc_text:
            if message[-1:] != "\n":
                message += "\n"
            message += record.exc_text
        if record.stack_info:
            if message[-1:] != "\n":
                message += "\n"
            message += formatter.formatStack(record.stack_info)
        return message



import datetime
import json
import logging
import sys

import pytz
from confluent_kafka import Producer

from .base import BaseLogHandler


def delivery_report(err, msg):
    pass


class KafkaHandler(BaseLogHandler):
    """
        A handler class which send data to logstash.
    """

    def __init__(self, log_server: str, topic: str, app_name: str = None, host_name: str = None,
                 extra_data: dict = None,
                 timezone: str = "Asia/Tehran"):
        super().__init__(app_name=app_name, host_name=host_name, extra_data=extra_data)
        self.log_server = log_server
        self.timezone = timezone
        self.topic = topic
        self.producer = Producer({'bootstrap.servers': self.log_server}) if self.log_server else None

    def _get_send_data(self, record: logging.LogRecord):
        send_data = super()._get_send_data(record=record)
        send_data["time"] = self.get_now_time_for_elastic()
        return send_data

    def send(self, send_data: dict):
        if self.producer is None:
            return
        self.producer.produce(self.topic, json.dumps(send_data).encode("utf-8"), callback=delivery_report)

    def emit(self, record: logging.LogRecord) -> None:
        if not self.log_server or not self.topic:
            return
        try:
            send_data = self._get_send_data(record=record)
            self.send(send_data=send_data)
        except RecursionError:  # See issue 36272
            raise
        except Exception as e:
            if sys.stderr:
                sys.stderr.write(f"error in send to kafka {self.log_server}. e: {e}")

    def get_now_time_for_elastic(self):
        tz = pytz.timezone(self.timezone)
        return datetime.datetime.now(tz=tz).strftime("%Y-%m-%dT%H:%M:%S%z")



import logging
import sys

import requests

from .base import BaseLogHandler


class LogstashHandler(BaseLogHandler):
    """
        A handler class which send data to logstash.
    """

    def __init__(self, log_server: str, app_name: str = None, host_name: str = None, extra_data: dict = None):
        super().__init__(app_name=app_name, host_name=host_name, extra_data=extra_data)
        self.log_server = log_server

    def send(self, send_data: dict):
        if not self.log_server:
            return
        urllib3_log = logging.getLogger("urllib3")
        urllib3_log.setLevel(logging.CRITICAL)
        urllib3_log.propagate = False
        headers = {"Content-Type": "application/json"}
        requests.post(self.log_server, json=send_data, headers=headers)

    def emit(self, record: logging.LogRecord) -> None:
        if not self.log_server:
            return
        send_data = None
        try:
            send_data = self._get_send_data(record=record)
            self.send(send_data=send_data)

        except RecursionError:  # See issue 36272
            raise
        except Exception as e:
            if sys.stderr:
                sys.stderr.write(f"error in send to logstash {self.log_server}. e: {e}, send_data: {send_data}")
